@@ -8,16 +8,16 @@ type ddpSelector struct {
 	profiles []string
 }
 
-// newDdpSelector returns a DeviceSelector interface to filter devices based on available DDP profile
-func newDdpSelector(profiles []string) types.DeviceSelector {
+// NewDdpSelector returns a DeviceSelector interface to filter devices based on available DDP profile
+func NewDdpSelector(profiles []string) types.DeviceSelector {
 	return &ddpSelector{profiles: profiles}
 }
 
-func (ds *ddpSelector) Filter(inDevices []types.PciNetDevice) []types.PciNetDevice {
-	filteredList := make([]types.PciNetDevice, 0)
+func (ds *ddpSelector) Filter(inDevices []types.PciDevice) []types.PciDevice {
+	filteredList := make([]types.PciDevice, 0)
 
 	for _, dev := range inDevices {
-		ddpProfile := dev.GetDDPProfiles()
+		ddpProfile := dev.(types.PciNetDevice).GetDDPProfiles()
 		if ddpProfile != "" && contains(ds.profiles, ddpProfile) {
 			filteredList = append(filteredList, dev)
 		}
